/***************************************************************************
 *   Copyright (C) 2008 by Thomas Schildknecht <danakil@gmail.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "panelspacer.h"

#include <QPainter>

#include <plasma/theme.h>
#include <KColorScheme>
#include <KConfigDialog>

#define SEPSTYLE_NONE 0
#define SEPSTYLE_LINE 1
#define SEPPOS_LEFTORTOP 0
#define SEPPOS_MIDDLE 1
#define SEPPOS_RIGHTORBOTTOM 2
#define MINIMALSIZE 6

//FIXME is there a way to get the blank space
//size in the panel ?
#define MAXFIXEDSIZE 600



PlasmaPanelSpacer::PlasmaPanelSpacer(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    m_spacerSize        = MINIMALSIZE+10;
    m_separatorStyle    = SEPSTYLE_LINE;
    m_separatorPosition = SEPPOS_LEFTORTOP;
    m_spacerStretch     = false;
    m_configureMode     = false;
    m_separatorWidth    = 2;
 
    resize(220, 80);
}

void PlasmaPanelSpacer::init()
{
    KConfigGroup cg = config();
    m_spacerSize = cg.readEntry("spacerSize", m_spacerSize);
    m_separatorStyle = cg.readEntry("separatorStyle", m_separatorStyle);
    m_separatorPosition = cg.readEntry("separatorPosition", m_separatorPosition);
    m_spacerStretch = cg.readEntry("separatorStretch", m_spacerStretch);

    if(formFactor()==Plasma::Horizontal || formFactor()==Plasma::Vertical) 
        setHasConfigurationInterface(true); 
}

void PlasmaPanelSpacer::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::SizeConstraint || constraints & Plasma::FormFactorConstraint) 
    {      
        if(formFactor() == Plasma::Horizontal) {          
            //setBackgroundHints(NoBackground);  
            setMinimumWidth(m_spacerSize);

            if(!m_spacerStretch) setMaximumWidth(m_spacerSize);
            else setMaximumWidth(-1);          
        }
        else if(formFactor() == Plasma::Vertical) {          
            //setBackgroundHints(NoBackground);
            setMinimumHeight(m_spacerSize);

            if(!m_spacerStretch) setMaximumHeight(m_spacerSize);   
            else setMaximumHeight(-1);
        }
    }
    update();
}


void PlasmaPanelSpacer::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED (option);

    int seppos=0;
    KColorScheme plasmaColorTheme = KColorScheme(QPalette::Active, KColorScheme::View, 
        Plasma::Theme::defaultTheme()->colorScheme());
    QColor sepcol = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);

    // warns the user if the plasmoid is on the desktop
    // and return
    if(formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter) { 
        p->setPen(QPen(sepcol));
        setHasConfigurationInterface(false);
        p->drawText(contentsRect,Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWordWrap,
                i18n("I'm a PanelSpacer so I'm quite useless here... Please put me in a panel !"));
        return;
    }

    // draw a frame around the applet area when
    // the config dialog is displayed
    if(m_configureMode) {
        sepcol.setAlphaF(0.08);
        QBrush appletAreaB(sepcol);
        p->setPen(Qt::NoPen);
        p->setBrush(appletAreaB);

        if(formFactor() == Plasma::Horizontal) {      
            if(m_spacerSize>=MINIMALSIZE)   // minimal size area           
                p->drawRect(0,0, m_spacerSize,contentsRect.bottom());            
            if(m_spacerStretch) {           // stretched area
                if(m_spacerSize>MINIMALSIZE)
                    p->drawRect(m_spacerSize+4,contentsRect.top(), 
                        contentsRect.right()-m_spacerSize-4,contentsRect.bottom());
                else p->drawRect(contentsRect); 
            }
        }
        else if(formFactor() == Plasma::Vertical) {
            if(m_spacerSize>=MINIMALSIZE)   // minimal size area           
                p->drawRect(0,0, contentsRect.right(), m_spacerSize);            
            if(m_spacerStretch) {           // stretched area
                if(m_spacerSize>MINIMALSIZE)
                    p->drawRect(0, m_spacerSize+4, contentsRect.right(),
                        contentsRect.bottom()-m_spacerSize-4);
                else p->drawRect(contentsRect); 
            }
        }
    }

    sepcol.setAlphaF(0.2);
    p->setPen(QPen(sepcol));

    switch (m_separatorStyle)
    {
        case SEPSTYLE_NONE : break;
        case SEPSTYLE_LINE :
            if(formFactor() == Plasma::Horizontal) {
                if(m_separatorPosition==SEPPOS_MIDDLE) seppos+=((int)contentsRect.width()/2);
                else if(m_separatorPosition==SEPPOS_RIGHTORBOTTOM)  
                    seppos+=(int)contentsRect.width()-m_separatorWidth;
                p->drawLine ( (int)contentsRect.left()+seppos, 0, 
                    (int)contentsRect.left()+seppos, (int)contentsRect.height() );
            }
            else if(formFactor() == Plasma::Vertical) {
                if(m_separatorPosition==SEPPOS_MIDDLE)      seppos+=((int)contentsRect.height()/2);
                else if(m_separatorPosition==SEPPOS_RIGHTORBOTTOM)  
                    seppos+=(int)contentsRect.height()-m_separatorWidth;
                p->drawLine ( 0, (int)contentsRect.top()+seppos, 
                    (int)contentsRect.width(), (int)contentsRect.top()+seppos );
            }
            break;    
    }
}

void PlasmaPanelSpacer::createConfigurationInterface(KConfigDialog *parent)
{
    m_configureMode= true;

    QWidget *widget = new QWidget();
    uiConfig.setupUi(widget);        
    parent->setButtons( KDialog::Ok | KDialog::Cancel );
    parent->addPage(widget, parent->windowTitle(), icon()); //parent->setMainWidget(widget);
    
    uiConfig.horizontalSliderSize->setMinimum(MINIMALSIZE);
    uiConfig.horizontalSliderSize->setMaximum(MAXFIXEDSIZE);
    uiConfig.horizontalSliderSize->setValue(m_spacerSize);

    if(m_spacerStretch) uiConfig.checkBoxStretch->setChecked(true);       
    else uiConfig.checkBoxStretch->setChecked(false);
                  
    uiConfig.comboBoxSeparatorStyle->addItem(i18n("None"));
    uiConfig.comboBoxSeparatorStyle->addItem(i18n("Line"));

    if(formFactor() == Plasma::Horizontal) {   
        uiConfig.comboBoxSeparatorPosition->addItem(i18n("Left"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18n("Middle"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18n("Right"));
    }
    else if(formFactor() == Plasma::Vertical){
        uiConfig.comboBoxSeparatorPosition->addItem(i18n("Top"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18n("Middle"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18n("Bottom"));
    }

    uiConfig.comboBoxSeparatorStyle->setCurrentIndex(m_separatorStyle);
    uiConfig.comboBoxSeparatorPosition->setCurrentIndex(m_separatorPosition);

    if(m_separatorStyle==SEPSTYLE_NONE) uiConfig.comboBoxSeparatorPosition->setEnabled(false);

    connect (parent, SIGNAL(okClicked()),       this, SLOT(configAccepted()));
    connect (parent, SIGNAL(finished () ),   this, SLOT(slotConfigClosed()));
    connect (uiConfig.checkBoxStretch, SIGNAL(toggled(bool)), this, SLOT(slotSpacerStretchChanged(bool)));
    connect (uiConfig.horizontalSliderSize, SIGNAL(sliderMoved(int)), this, SLOT(slotSpacerSizeChanged(int)));
    connect (uiConfig.comboBoxSeparatorStyle, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSeparatorStyleChanged(int)));
    connect (uiConfig.comboBoxSeparatorPosition, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSeparatorPositionChanged(int)));

    update();
}

void PlasmaPanelSpacer::configAccepted()
{ 
    KConfigGroup cg = config();    
    cg.writeEntry("spacerSize", uiConfig.horizontalSliderSize->value());
    cg.writeEntry("separatorStyle", uiConfig.comboBoxSeparatorStyle->currentIndex());
    cg.writeEntry("separatorPosition", uiConfig.comboBoxSeparatorPosition->currentIndex());
    cg.writeEntry("separatorStretch", uiConfig.checkBoxStretch->isChecked());
    emit configNeedsSaving();
}

void PlasmaPanelSpacer::slotConfigClosed()
{
    m_configureMode=false;
    update();
}

void PlasmaPanelSpacer::slotSpacerStretchChanged(bool checkstate)
{
    m_spacerStretch=checkstate;
    updateConstraints();
}

void PlasmaPanelSpacer::slotSeparatorPositionChanged(int index)
{
    m_separatorPosition=index;
    updateConstraints();
}

void PlasmaPanelSpacer::slotSeparatorStyleChanged(int index)
{
    m_separatorStyle=index;

    if(index==SEPSTYLE_NONE) uiConfig.comboBoxSeparatorPosition->setEnabled(false);
    else uiConfig.comboBoxSeparatorPosition->setEnabled(true);

    updateConstraints();
}

void PlasmaPanelSpacer::slotSpacerSizeChanged(int size)
{
    m_spacerSize=size;
    updateConstraints();
}

#include "panelspacer.moc"
